# Copyright (c) 2014 NetApp, Inc.
# All rights reserved.

# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from django.conf import settings
from django.forms import ValidationError
from django.urls import reverse
from django.utils.translation import gettext_lazy as _
from horizon import exceptions
from horizon import forms
from horizon import messages
from horizon.utils.memoized import memoized

from manila_ui.api import manila
from manila_ui.dashboards import utils
from manila_ui import features
from manilaclient.common.apiclient import exceptions as m_exceptions
import time


class CreateForm(forms.SelfHandlingForm):
    name = forms.CharField(max_length="255", label=_("Share Name"))
    description = forms.CharField(
        label=_("Description"), required=False,
        widget=forms.Textarea(attrs={'rows': 3}))
    share_proto = forms.ChoiceField(label=_("Share Protocol"), required=True)
    size = forms.IntegerField(min_value=1, label=_("Size (GiB)"))
    share_type = forms.ChoiceField(
        label=_("Share Type"), required=True,
        widget=forms.Select(
            attrs={'class': 'switchable', 'data-slug': 'sharetype'}))
    availability_zone = forms.ChoiceField(
        label=_("Availability Zone"),
        required=False)

    def __init__(self, request, *args, **kwargs):
        super(CreateForm, self).__init__(request, *args, **kwargs)
        # NOTE(vkmc): choose only those share protocols that are enabled
        # FIXME(vkmc): this should be better implemented by having a
        # capabilities endpoint on the control plane.
        manila_features = getattr(settings, 'OPENSTACK_MANILA_FEATURES', {})
        self.enabled_share_protocols = manila_features.get(
            'enabled_share_protocols',
            ['NFS', 'CIFS', 'GlusterFS', 'HDFS', 'CephFS', 'MapRFS'])
        self.enable_public_shares = manila_features.get(
            'enable_public_shares', True)
        share_networks = manila.share_network_list(request)
        share_types = manila.share_type_list(request)
        self.fields['share_type'].choices = (
            [(utils.transform_dashed_name(st.name), st.name) for st in
             share_types]
        )

        availability_zones = manila.availability_zone_list(request)
        self.fields['availability_zone'].choices = (
            [("", "")] + [(az.name, az.name) for az in availability_zones])

        if features.is_share_groups_enabled():
            share_groups = manila.share_group_list(request)
            self.fields['sg'] = forms.ChoiceField(
                label=_("Share Group"),
                choices=[("", "")] + [(sg.id, sg.name or sg.id)
                                      for sg in share_groups],
                required=False)

        self.sn_field_name_prefix = 'share-network-choices-'
        for st in share_types:
            extra_specs = st.get_keys()
            dhss = extra_specs.get('driver_handles_share_servers')
            # NOTE(vponomaryov): Set and tie share-network field only for
            # share types with enabled handling of share servers.
            if (isinstance(dhss, str) and dhss.lower() in ['true', '1']):
                sn_choices = (
                    [(sn.id, sn.name or sn.id) for sn in share_networks])
                sn_field_name = (
                    self.sn_field_name_prefix +
                    utils.transform_dashed_name(st.name)
                )
                sn_field = forms.ChoiceField(
                    label=_("Share Network"), required=True,
                    choices=sn_choices,
                    widget=forms.Select(attrs={
                        'class': 'switched',
                        'data-switch-on': 'sharetype',
                        'data-sharetype-%s' % utils.transform_dashed_name(
                            st.name): _("Share Network"),
                    }))
                self.fields[sn_field_name] = sn_field

        self.fields['share_source_type'] = forms.ChoiceField(
            label=_("Share Source"), required=False,
            widget=forms.Select(
                attrs={'class': 'switchable', 'data-slug': 'source'}))
        self.fields['snapshot'] = forms.ChoiceField(
            label=_("Use snapshot as a source"),
            widget=forms.fields.SelectWidget(
                attrs={'class': 'switched',
                       'data-switch-on': 'source',
                       'data-source-snapshot': _('Snapshot')},
                data_attrs=('size', 'name'),
                transform=lambda x: "%s (%sGiB)" % (x.name, x.size)),
            required=False)
        self.fields['metadata'] = forms.CharField(
            label=_("Metadata"), required=False,
            widget=forms.Textarea(attrs={'rows': 4}))

        if self.enable_public_shares:
            self.fields['is_public'] = forms.BooleanField(
                label=_("Make visible to users from all projects"),
                required=False)

        self.fields['share_proto'].choices = [(sp, sp) for sp in
                                              self.enabled_share_protocols]
        if ("snapshot_id" in request.GET or
                kwargs.get("data", {}).get("snapshot")):
            try:
                snapshot = self.get_snapshot(
                    request,
                    request.GET.get("snapshot_id",
                                    kwargs.get("data", {}).get("snapshot")))
                self.fields['name'].initial = snapshot.name
                self.fields['size'].initial = snapshot.size
                self.fields['snapshot'].choices = ((snapshot.id, snapshot),)
                try:
                    # Set the share type and az from the original share
                    orig_share = manila.share_get(request, snapshot.share_id)
                    self.fields['share_type'].initial = (
                        orig_share.share_type_name)
                    self.fields['availability_zone'].initial = (
                        orig_share.availability_zone)
                except Exception:
                    pass
                self.fields['size'].help_text = _(
                    'Share size must be equal to or greater than the snapshot '
                    'size (%sGiB)') % snapshot.size
                del self.fields['share_source_type']
            except Exception:
                exceptions.handle(request,
                                  _('Unable to load the specified snapshot.'))
        else:
            source_type_choices = []

            try:
                snapshot_list = manila.share_snapshot_list(request)
                snapshots = [s for s in snapshot_list
                             if s.status == 'available']
                if snapshots:
                    source_type_choices.append(("snapshot",
                                                _("Snapshot")))
                    choices = [('', _("Choose a snapshot"))] + \
                              [(s.id, s) for s in snapshots]
                    self.fields['snapshot'].choices = choices
                else:
                    del self.fields['snapshot']
            except Exception:
                exceptions.handle(request, _("Unable to retrieve "
                                             "share snapshots."))

            if source_type_choices:
                choices = ([('no_source_type',
                             _("No source, empty share"))] +
                           source_type_choices)
                self.fields['share_source_type'].choices = choices
            else:
                del self.fields['share_source_type']

    def clean(self):
        cleaned_data = super(CreateForm, self).clean()
        form_errors = list(self.errors)

        chosen_share_type = cleaned_data.get('share_type')
        if chosen_share_type:
            # NOTE(vponomaryov): skip errors for share-network fields that are
            # not related to chosen share type.
            for error in form_errors:
                st_name = error.split(self.sn_field_name_prefix)[-1]
                if (error.startswith(self.sn_field_name_prefix) and
                        st_name != chosen_share_type):
                    cleaned_data[error] = 'Not set'
                    self.errors.pop(error, None)

            cleaned_data['share_network'] = cleaned_data.get(
                self.sn_field_name_prefix + cleaned_data.get('share_type'))

        return cleaned_data

    def handle(self, request, data):
        try:
            snapshot_id = None
            source_type = data.get('share_source_type', None)
            share_network = data.get('share_network', None)
            if (data.get("snapshot", None) and
                    source_type in [None, 'snapshot']):
                # Create from Snapshot
                snapshot = self.get_snapshot(request,
                                             data["snapshot"])
                snapshot_id = snapshot.id
                if (data['size'] < snapshot.size):
                    error_message = _('The share size cannot be less than the '
                                      'snapshot size (%sGiB)') % snapshot.size
                    raise ValidationError(error_message)
            else:
                data['size'] = int(data['size'])

            metadata = {}
            try:
                set_dict, unset_list = utils.parse_str_meta(data['metadata'])
                if unset_list:
                    msg = _("Expected only pairs of key=value.")
                    raise ValidationError(message=msg)
                metadata = set_dict
            except ValidationError as e:
                self.api_error(e.messages[0])
                return False
            is_public = self.enable_public_shares and data['is_public']
            share = manila.share_create(
                request,
                size=data['size'],
                name=data['name'],
                description=data['description'],
                proto=data['share_proto'],
                share_network=share_network,
                snapshot_id=snapshot_id,
                share_type=utils.transform_dashed_name(data['share_type']),
                is_public=is_public,
                metadata=metadata,
                availability_zone=data.get('availability_zone') or None,
                share_group_id=data.get('sg') or None,
            )
            message = _('Creating share "%s"') % data['name']
            messages.success(request, message)
            return share
        except ValidationError as e:
            self.api_error(e.messages[0])
        except m_exceptions.BadRequest as e:
            self.api_error(_("Unable to create share. %s") % e.message)
        except Exception:
            exceptions.handle(request, ignore=True)
            self.api_error(_("Unable to create share."))
        return False

    @memoized
    def get_snapshot(self, request, id):
        return manila.share_snapshot_get(request, id)


class UpdateForm(forms.SelfHandlingForm):
    name = forms.CharField(max_length="255", label=_("Share Name"))
    description = forms.CharField(widget=forms.Textarea,
                                  label=_("Description"), required=False)
    is_public = forms.BooleanField(
        label=_("Make visible to users from all projects"),
        required=False)

    def __init__(self, request, *args, **kwargs):
        super(UpdateForm, self).__init__(request, *args, **kwargs)
        manila_features = getattr(settings, 'OPENSTACK_MANILA_FEATURES', {})
        self.enable_public_shares = manila_features.get(
            'enable_public_shares', True)
        if not self.enable_public_shares:
            self.fields.pop('is_public')

    def handle(self, request, data):
        share_id = self.initial['share_id']
        is_public = data['is_public'] if self.enable_public_shares else False
        try:
            share = manila.share_get(self.request, share_id)
            manila.share_update(
                request, share, data['name'], data['description'],
                is_public=is_public)
            message = _('Updating share "%s"') % data['name']
            messages.success(request, message)
            return True
        except Exception:
            redirect = reverse("horizon:project:shares:index")
            exceptions.handle(request,
                              _('Unable to update share.'),
                              redirect=redirect)


class UpdateMetadataForm(forms.SelfHandlingForm):
    metadata = forms.CharField(widget=forms.Textarea,
                               label=_("Metadata"), required=False)

    def __init__(self, *args, **kwargs):
        super(UpdateMetadataForm, self).__init__(*args, **kwargs)
        meta_str = ""
        for k, v in self.initial["metadata"].items():
            meta_str += "%s=%s\r\n" % (k, v)
        self.initial["metadata"] = meta_str

    def handle(self, request, data):
        share_id = self.initial['share_id']
        try:
            share = manila.share_get(self.request, share_id)
            set_dict, unset_list = utils.parse_str_meta(data['metadata'])
            if set_dict:
                manila.share_set_metadata(request, share, set_dict)
            if unset_list:
                manila.share_delete_metadata(request, share, unset_list)
            message = _('Updating share metadata "%s"') % share.name
            messages.success(request, message)
            return True
        except ValidationError as e:
            self.api_error(e.messages[0])
            return False
        except Exception:
            redirect = reverse("horizon:project:shares:index")
            exceptions.handle(request,
                              _('Unable to update share metadata.'),
                              redirect=redirect)


class AddRule(forms.SelfHandlingForm):
    access_type = forms.ChoiceField(
        label=_("Access Type"), required=True,
        choices=(('ip', 'ip'), ('user', 'user'), ('cert', 'cert'),
                 ('cephx', 'cephx')))
    access_level = forms.ChoiceField(
        label=_("Access Level"), required=True,
        choices=(('rw', 'read-write'), ('ro', 'read-only'),))
    access_to = forms.CharField(
        label=_("Access To"), max_length="255", required=True)
    metadata = forms.CharField(
        label=_("Metadata"), required=False,
        widget=forms.Textarea(attrs={'rows': 4}))

    def handle(self, request, data):
        share_id = self.initial['share_id']
        metadata = {}
        try:
            set_dict, unset_list = utils.parse_str_meta(data['metadata'])
            if unset_list:
                msg = _("Expected only pairs of key=value.")
                raise ValidationError(message=msg)
            metadata = set_dict
        except ValidationError as e:
            self.api_error(e.messages[0])
            return False
        try:
            manila.share_allow(
                request, share_id,
                access_to=data['access_to'],
                access_type=data['access_type'],
                access_level=data['access_level'],
                metadata=metadata)
            message = _('Creating rule for "%s"') % data['access_to']
            messages.success(request, message)
            return True
        except Exception:
            redirect = reverse("horizon:project:shares:manage_rules",
                               args=[self.initial['share_id']])
            exceptions.handle(
                request, _('Unable to add rule.'), redirect=redirect)


class UpdateRuleMetadataForm(forms.SelfHandlingForm):
    metadata = forms.CharField(widget=forms.Textarea,
                               label=_("Metadata"), required=False)

    def __init__(self, *args, **kwargs):
        super(UpdateRuleMetadataForm, self).__init__(*args, **kwargs)
        rule_metadata = utils.metadata_to_str(
            self.initial["metadata"]
        ).replace('<br/>', '\r\n')
        self.initial["metadata"] = rule_metadata

    def handle(self, request, data):
        rule_id = self.initial['rule_id']
        try:
            rule = manila.share_rule_get(self.request, rule_id)
            set_dict, unset_list = utils.parse_str_meta(data['metadata'])
            if set_dict:
                manila.share_rule_set_metadata(request, rule, set_dict)
            if unset_list:
                manila.share_rule_unset_metadata(request, rule, unset_list)
            message = _('Updating share access rule metadata ')
            messages.success(request, message)
            return True
        except ValidationError as e:
            self.api_error(e.messages[0])
            return False
        except Exception:
            redirect = reverse("horizon:project:shares:manage_rules",
                               args=[self.initial['share_id']])
            exceptions.handle(request,
                              _('Unable to update rule metadata.'),
                              redirect=redirect)


class ResizeForm(forms.SelfHandlingForm):
    name = forms.CharField(
        max_length="255", label=_("Share Name"),
        widget=forms.TextInput(attrs={'readonly': 'readonly'}),
        required=False,
    )

    orig_size = forms.IntegerField(
        label=_("Current Size (GiB)"),
        widget=forms.TextInput(attrs={'readonly': 'readonly'}),
        required=False,
    )

    new_size = forms.IntegerField(
        label=_("New Size (GiB)")
    )

    def clean(self):
        cleaned_data = super(ResizeForm, self).clean()
        new_size = cleaned_data.get('new_size')
        orig_size = self.initial['orig_size']

        if new_size == orig_size:
            message = _("New size must be different than the existing size")
            self._errors["new_size"] = self.error_class([message])
            return cleaned_data

        if new_size <= 0:
            message = _("New size should not be less than or equal to zero")
            self._errors["new_size"] = self.error_class([message])
            return cleaned_data

        usages = manila.tenant_absolute_limits(self.request)
        availableGB = (usages['maxTotalShareGigabytes'] -
                       usages['totalShareGigabytesUsed'])
        if availableGB < (new_size - orig_size):
            message = _('Share cannot be extended to %(req)iGiB as '
                        'you only have %(avail)iGiB of your quota '
                        'available.')
            params = {'req': new_size, 'avail': availableGB + orig_size}
            self._errors["new_size"] = self.error_class([message % params])
        return cleaned_data

    def handle(self, request, data):
        share_id = self.initial['share_id']
        new_size = data['new_size']
        orig_size = data['orig_size']
        try:
            manila.share_resize(request, share_id, new_size, orig_size)
            return self.check_size(request, share_id, new_size)
        except Exception:
            redirect = reverse("horizon:project:shares:index")
            exceptions.handle(request, _(
                'Unable to resize share.'), redirect=redirect)

    def check_size(self, request, share_id, new_size):
        share = manila.share_get(request, share_id)
        timeout = 30
        interval = 0.35
        time_elapsed = 0
        while share.status != 'available':
            time.sleep(interval)
            time_elapsed += interval
            share = manila.share_get(request, share_id)
            if time_elapsed > timeout:
                raise exceptions.WorkflowError(_(
                    "The operation timed out while resizing. "
                    "Please try again."))

        if share.size == new_size:
            message = _('Resized share "%s"') % share.name
            messages.success(request, message)
            return True
        raise exceptions.WorkflowError(_(
            "Unable to resize share. "))


class RevertForm(forms.SelfHandlingForm):
    """Form for reverting a share to a snapshot."""

    snapshot = forms.ChoiceField(
        label=_("Snapshot"),
        required=True,
        widget=forms.Select(
            attrs={'class': 'switchable', 'data-slug': 'share_snapshot'}))

    def __init__(self, req, *args, **kwargs):
        super(self.__class__, self).__init__(req, *args, **kwargs)
        # NOTE(vponomaryov): manila client does not allow to filter snapshots
        # using "created_at" field, so, we need to get all snapshots of a share
        # and do filtering here.
        search_opts = {'share_id': self.initial['share_id']}
        snapshots = manila.share_snapshot_list(req, search_opts=search_opts)
        amount_of_snapshots = len(snapshots)
        if amount_of_snapshots < 1:
            self.fields['snapshot'].choices = [("", "")]
        else:
            snapshot = snapshots[0]
            if amount_of_snapshots > 1:
                for s in snapshots[1:]:
                    if s.created_at > snapshot.created_at:
                        snapshot = s
            self.fields['snapshot'].choices = [
                (snapshot.id, snapshot.name or snapshot.id)]

    def handle(self, request, data):
        share_id = self.initial['share_id']
        snapshot_id = data['snapshot']
        try:
            manila.share_revert(request, share_id, snapshot_id)
            message = _('Share "%(s)s" has been reverted to "%(ss)s" snapshot '
                        'successfully') % {'s': share_id, 'ss': snapshot_id}
            messages.success(request, message)
            return True
        except Exception:
            redirect = reverse("horizon:project:shares:index")
            exceptions.handle(
                request,
                _('Unable to revert share to the snapshot.'),
                redirect=redirect)



# Copyright (c) 2016 Mirantis, Inc.
# All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from django.urls import reverse
from django.utils.translation import gettext_lazy as _

from horizon import exceptions
from horizon import forms

from manila_ui.api import manila
from manila_ui.dashboards.admin.shares.replicas import (
    tables as replicas_tables)
from manila_ui.dashboards.admin.shares.replicas import forms as replicas_forms
from manila_ui.dashboards.admin.shares.replicas import tabs as replicas_tabs
from manila_ui.dashboards.project.shares.replicas import (
    views as project_replica_views)


class ManageReplicasView(project_replica_views.ManageReplicasView):
    table_class = replicas_tables.ReplicasTable
    template_name = 'admin/shares/replicas/manage_replicas.html'
    _redirect_url = 'horizon:admin:shares:index'


class DetailReplicaView(project_replica_views.DetailReplicaView):
    tab_group_class = replicas_tabs.ReplicaDetailTabs
    template_name = 'admin/shares/replicas/detail.html'
    _redirect_url = 'horizon:admin:shares:index'


class ResyncReplicaView(forms.ModalFormView):
    form_class = replicas_forms.ResyncReplicaForm
    form_id = "resync_replica"
    template_name = 'admin/shares/replicas/resync_replica.html'
    modal_header = _("Resync Replica")
    modal_id = "resync_replica_modal"
    submit_label = _("Resync")
    submit_url = "horizon:admin:shares:resync_replica"
    success_url = 'horizon:admin:shares:manage_replicas'
    page_title = _("Resync Replica")

    def get_success_url(self):
        return reverse(self.success_url, args=[self.get_object().share_id])

    def get_object(self):
        if not hasattr(self, "_object"):
            replica_id = self.kwargs["replica_id"]
            try:
                self._object = manila.share_replica_get(
                    self.request, replica_id)
            except Exception:
                msg = _("Unable to retrieve replica '%s'.") % replica_id
                url = reverse('horizon:admin:shares:index')
                exceptions.handle(self.request, msg, redirect=url)
        return self._object

    def get_context_data(self, **kwargs):
        context = super(self.__class__, self).get_context_data(**kwargs)
        context['replica_id'] = self.kwargs['replica_id']
        args = (context['replica_id'],)
        context['submit_url'] = reverse(self.submit_url, args=args)
        return context

    def get_initial(self):
        return {'replica_id': self.kwargs["replica_id"]}


class ResetReplicaStatusView(forms.ModalFormView):
    form_class = replicas_forms.ResetReplicaStatusForm
    form_id = "reset_replica_status"
    template_name = 'admin/shares/replicas/reset_replica_status.html'
    modal_header = _("Reset Replica Status")
    modal_id = "reset_replica_status_modal"
    submit_label = _("Reset status")
    submit_url = "horizon:admin:shares:reset_replica_status"
    success_url = 'horizon:admin:shares:manage_replicas'
    page_title = _("Reset Replica Status")

    def get_success_url(self):
        return reverse(self.success_url, args=[self.get_object().share_id])

    def get_object(self):
        if not hasattr(self, "_object"):
            replica_id = self.kwargs["replica_id"]
            try:
                self._object = manila.share_replica_get(
                    self.request, replica_id)
            except Exception:
                msg = _("Unable to retrieve replica '%s'.") % replica_id
                url = reverse('horizon:admin:shares:index')
                exceptions.handle(self.request, msg, redirect=url)
        return self._object

    def get_context_data(self, **kwargs):
        context = super(self.__class__, self).get_context_data(**kwargs)
        context['replica_id'] = self.kwargs['replica_id']
        args = (context['replica_id'],)
        context['submit_url'] = reverse(self.submit_url, args=args)
        return context

    def get_initial(self):
        return {'replica_id': self.kwargs["replica_id"]}


class ResetReplicaStateView(forms.ModalFormView):
    form_class = replicas_forms.ResetReplicaStateForm
    form_id = "reset_replica_state"
    template_name = 'admin/shares/replicas/reset_replica_state.html'
    modal_header = _("Reset Replica State")
    modal_id = "reset_replica_state_modal"
    submit_label = _("Reset state")
    submit_url = "horizon:admin:shares:reset_replica_state"
    success_url = 'horizon:admin:shares:manage_replicas'
    page_title = _("Reset Replica State")

    def get_success_url(self):
        return reverse(self.success_url, args=[self.get_object().share_id])

    def get_object(self):
        if not hasattr(self, "_object"):
            replica_id = self.kwargs["replica_id"]
            try:
                self._object = manila.share_replica_get(
                    self.request, replica_id)
            except Exception:
                msg = _("Unable to retrieve replica '%s'.") % replica_id
                url = reverse('horizon:admin:shares:index')
                exceptions.handle(self.request, msg, redirect=url)
        return self._object

    def get_context_data(self, **kwargs):
        context = super(self.__class__, self).get_context_data(**kwargs)
        context['replica_id'] = self.kwargs['replica_id']
        args = (context['replica_id'],)
        context['submit_url'] = reverse(self.submit_url, args=args)
        return context

    def get_initial(self):
        return {'replica_id': self.kwargs["replica_id"]}



# Copyright (c) 2017 Mirantis, Inc.
# All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from django.urls import reverse
from django.utils.translation import gettext_lazy as _
from horizon import exceptions
from horizon import forms
from horizon import messages

from manila_ui.api import manila


class ResetShareGroupStatusForm(forms.SelfHandlingForm):
    status = forms.ChoiceField(
        label=_("Status"),
        required=True,
        choices=(
            ('available', 'available'),
            ('error', 'error'),
        )
    )

    def handle(self, request, data):
        sg_id = self.initial['share_group_id']
        try:
            manila.share_group_reset_state(request, sg_id, data["status"])
            message = _(
                "Reseting share group ('%(id)s') status from '%(from)s' "
                "to '%(to)s'.") % {
                    "id": (self.initial['share_group_name'] or
                           self.initial['share_group_id']),
                    "from": self.initial['share_group_status'],
                    "to": data["status"]}
            messages.success(request, message)
            return True
        except Exception:
            redirect = reverse("horizon:admin:share_groups:index")
            exceptions.handle(
                request,
                _("Unable to reset status of share group '%s'.") % sg_id,
                redirect=redirect)
        return False



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from django.template.defaultfilters import title
from django.urls import reverse
from django.utils.translation import gettext_lazy as _
from django.utils.translation import ngettext_lazy
from django.utils.translation import pgettext_lazy
from horizon import tables

from manila_ui.api import manila


class DeleteShareServer(tables.DeleteAction):
    policy_rules = (("share", "share_server:delete"),)

    @staticmethod
    def action_present(count):
        return ngettext_lazy(
            u"Delete Share Server",
            u"Delete Share Server",
            count
        )

    @staticmethod
    def action_past(count):
        return ngettext_lazy(
            u"Deleted Share Server",
            u"Deleted Share Server",
            count
        )

    def delete(self, request, obj_id):
        manila.share_server_delete(request, obj_id)

    def allowed(self, request, share_serv):
        if share_serv:
            share_search_opts = {'share_server_id': share_serv.id}
            shares_list = manila.share_list(
                request, search_opts=share_search_opts)
            if shares_list:
                return False
            return share_serv.status not in ("deleting", "creating")
        return True


class UpdateShareServerRow(tables.Row):
    ajax = True

    def get_data(self, request, share_serv_id):
        share_serv = manila.share_server_get(request, share_serv_id)
        return share_serv


class ShareServersTable(tables.DataTable):
    STATUS_CHOICES = (
        ("active", True),
        ("deleting", None),
        ("creating", None),
        ("error", False),
    )
    STATUS_DISPLAY_CHOICES = (
        ("in-use", pgettext_lazy("Current status of share server", u"In-use")),
        ("active", pgettext_lazy("Current status of share server", u"Active")),
        ("creating", pgettext_lazy("Current status of share server",
                                   u"Creating")),
        ("error", pgettext_lazy("Current status of share server",
                                u"Error")),
    )
    uid = tables.Column(
        "id", verbose_name=_("Id"),
        link="horizon:admin:share_servers:share_server_detail")
    host = tables.Column("host", verbose_name=_("Host"))
    project = tables.Column("project_name", verbose_name=_("Project"))

    def get_share_server_link(share_serv):
        if hasattr(share_serv, 'share_network_id'):
            return reverse("horizon:admin:share_networks:share_network_detail",
                           args=(share_serv.share_network_id,))
        else:
            return None

    share_net_name = tables.Column(
        "share_network_name", verbose_name=_("Share Network"),
        link=get_share_server_link)
    status = tables.Column(
        "status", verbose_name=_("Status"),
        status=True, filters=(title,), status_choices=STATUS_CHOICES,
        display_choices=STATUS_DISPLAY_CHOICES)

    def get_object_display(self, share_server):
        return str(share_server.id)

    def get_object_id(self, share_server):
        return str(share_server.id)

    class Meta(object):
        name = "share_servers"
        status_columns = ["status"]
        verbose_name = _("Share Server")
        table_actions = (
            tables.NameFilterAction,
            DeleteShareServer,
        )
        row_class = UpdateShareServerRow
        row_actions = (
            DeleteShareServer,
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from django.utils.translation import gettext_lazy as _
from horizon import tables

import manila_ui.dashboards.project.share_networks.tables as sn_tables


class ShareNetworksFilterAction(tables.FilterAction):
    filter_type = "server"
    filter_choices = (
        ('name', _("Name "), True),
        ('description', _("Description "), True),
    )


class ShareNetworksTable(tables.DataTable):
    name = tables.WrappingColumn(
        "name", verbose_name=_("Name"),
        link="horizon:admin:share_networks:share_network_detail")
    description = tables.WrappingColumn(
        "description", verbose_name=_("Description"))
    project = tables.Column("project_name", verbose_name=_("Project"))

    def get_object_display(self, share_network):
        return share_network.name or str(share_network.id)

    def get_object_id(self, share_network):
        return str(share_network.id)

    class Meta(object):
        name = "share_networks"
        verbose_name = _("Share Networks")
        table_actions = (
            ShareNetworksFilterAction,
            sn_tables.Delete,
        )
        row_class = sn_tables.UpdateRow
        row_actions = (
            sn_tables.Delete,
        )



# Copyright (c) 2014 NetApp, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from django.urls import reverse
from neutronclient.client import exceptions
from openstack_auth import policy
from openstack_dashboard import api
from oslo_utils import timeutils
from unittest import mock

from manila_ui.api import manila as api_manila
from manila_ui.dashboards import utils
from manila_ui.tests.dashboards.project import test_data
from manila_ui.tests import helpers as test

INDEX_URL = reverse('horizon:project:share_networks:index')


class ShareNetworksViewTests(test.TestCase):
    class FakeAZ(object):
        def __init__(self, name, id):
            self.name = name
            self.id = id
            self.created_at = timeutils.utcnow()

    def test_create_share_network(self):
        share_net = test_data.active_share_network

        url = reverse('horizon:project:share_networks:share_network_create')
        neutron_net_id = self.networks.first().id
        sanitized_net_id = utils.transform_dashed_name(neutron_net_id)
        formData = {
            'share_network_name': 'new_share_network',
            'share_network_description': 'This is test share network',
            'method': 'CreateForm',
            'neutron_net_id': utils.transform_dashed_name(neutron_net_id),
            'availability_zone': 'fake_az',
            f'subnet-choices-{sanitized_net_id}':
                self.networks.first().subnets[0].id,
        }

        self.mock_object(
            api.neutron, "subnet_list",
            mock.Mock(return_value=self.subnets.list()))
        self.mock_object(
            api.neutron, "network_list",
            mock.Mock(return_value=self.networks.list()))
        self.mock_object(
            api_manila, "share_network_create",
            mock.Mock(return_value=share_net))
        self.mock_object(
            api_manila, "availability_zone_list",
            mock.Mock(return_value=[self.FakeAZ('fake_az', 'fake_az')])
        )

        res = self.client.post(url, formData)

        self.assertNoFormErrors(res)
        self.assertMessageCount(error=0, warning=0)
        self.assertRedirectsNoFollow(res, INDEX_URL)
        api_manila.share_network_create.assert_called_once_with(
            mock.ANY, name=formData['share_network_name'],
            neutron_net_id=neutron_net_id,
            neutron_subnet_id=self.networks.first().subnets[0].id,
            description=formData['share_network_description'],
            availability_zone='fake_az')
        api_manila.availability_zone_list.assert_called_once_with(mock.ANY)
        api.neutron.network_list.assert_called_once_with(mock.ANY)
        api.neutron.subnet_list.assert_has_calls([
            mock.call(mock.ANY, network_id=network.id)
            for network in self.networks.list()
        ], any_order=True)

    def test_delete_share_network(self):
        share_network = test_data.inactive_share_network
        formData = {'action': 'share_networks__delete__%s' % share_network.id}
        self.mock_object(api_manila, "share_network_delete")
        self.mock_object(
            api_manila, "share_network_list",
            mock.Mock(return_value=[
                test_data.active_share_network, share_network]))

        res = self.client.post(INDEX_URL, formData)

        self.assertRedirectsNoFollow(res, INDEX_URL)
        api_manila.share_network_list.assert_called_once_with(
            mock.ANY, detailed=True)
        api_manila.share_network_delete.assert_called_once_with(
            mock.ANY, share_network.id)

    def test_detail_view(self):
        share_net = test_data.active_share_network
        sec_service = test_data.sec_service
        share_network_subnets = share_net.share_network_subnets
        self.mock_object(
            api_manila, "share_server_list", mock.Mock(return_value=[]))
        self.mock_object(
            api_manila, "share_network_get", mock.Mock(return_value=share_net))
        self.mock_object(
            api_manila, "share_network_security_service_list",
            mock.Mock(return_value=[sec_service]))
        network = self.networks.first()
        subnet = self.subnets.first()

        self.mock_object(
            api.neutron, "network_get", mock.Mock(return_value=network))
        self.mock_object(
            api.neutron, "subnet_get", mock.Mock(return_value=subnet))
        self.mock_object(
            api_manila, "availability_zone_list",
            mock.Mock(return_value=[self.FakeAZ('fake_az', 'fake_az')])
        )
        url = reverse('horizon:project:share_networks:share_network_detail',
                      args=[share_net.id])

        res = self.client.get(url)

        self.assertNoMessages()
        self.assertContains(res, "<h1>Share Network Details: %s</h1>"
                                 % share_net.name,
                            1, 200)
        self.assertContains(res, "<dd>%s</dd>" % share_net.name, 1, 200)
        self.assertContains(res, "<dd>%s</dd>" % share_net.id, 1, 200)
        for sub in share_network_subnets:
            self.assertContains(res, "<a href=\"/project/networks"
                                "/%s/detail\">%s</a>" % (
                                    sub['neutron_net_id'],
                                    network.name), 1, 200)
            self.assertContains(res, "<a href=\"/project/networks/subnets"
                                     "/%s/detail\">%s</a>" % (
                                         sub['neutron_subnet_id'],
                                         subnet['name']), 1, 200)
        network_get_calls = [mock.call(mock.ANY, sub['neutron_net_id'])
                             for sub in share_network_subnets]
        subnet_get_calls = [mock.call(mock.ANY, sub['neutron_subnet_id'])
                            for sub in share_network_subnets]

        api.neutron.network_get.assert_has_calls(network_get_calls,
                                                 any_order=True)
        api.neutron.subnet_get.assert_has_calls(subnet_get_calls,
                                                any_order=True)
        self.assertContains(res, "<a href=\"/project/security_services"
                                 "/%s\">%s</a>" % (sec_service.id,
                                                   sec_service.name), 1, 200)
        api_manila.share_network_security_service_list.assert_called_once_with(
            mock.ANY, share_net.id)
        api_manila.share_server_list.assert_called_once_with(
            mock.ANY, search_opts={'share_network_id': share_net.id})
        api_manila.share_network_get.assert_called_once_with(
            mock.ANY, share_net.id)

    def test_detail_view_network_not_found(self):
        share_net = test_data.active_share_network
        sec_service = test_data.sec_service
        share_network_subnets = share_net.share_network_subnets
        url = reverse('horizon:project:share_networks:share_network_detail',
                      args=[share_net.id])
        self.mock_object(
            api_manila, "share_server_list", mock.Mock(return_value=[]))
        self.mock_object(
            api_manila, "share_network_get", mock.Mock(return_value=share_net))
        self.mock_object(
            api_manila, "share_network_security_service_list",
            mock.Mock(return_value=[sec_service]))
        self.mock_object(
            api.neutron, "network_get", mock.Mock(
                side_effect=exceptions.NeutronClientException('fake', 500)))
        self.mock_object(
            api.neutron, "subnet_get", mock.Mock(
                side_effect=exceptions.NeutronClientException('fake', 500)))
        self.mock_object(
            api_manila, "availability_zone_list",
            mock.Mock(return_value=[])
        )
        res = self.client.get(url)

        self.assertContains(res, "<h1>Share Network Details: %s</h1>"
                                 % share_net.name,
                            1, 200)
        self.assertContains(res, "<dd>%s</dd>" % share_net.name, 1, 200)
        self.assertContains(res, "<dd>%s</dd>" % share_net.id, 1, 200)
        for sub in share_network_subnets:
            self.assertNotContains(res, "<dd>%s</dd>" % sub['neutron_net_id'])
            self.assertNotContains(res,
                                   "<dd>%s</dd>" % sub['neutron_subnet_id'])
        network_get_calls = [mock.call(mock.ANY, sub['neutron_net_id']
                                       ) for sub in share_network_subnets]
        subnet_get_calls = [mock.call(mock.ANY, sub['neutron_subnet_id']
                                      ) for sub in share_network_subnets]

        api.neutron.network_get.assert_has_calls(network_get_calls,
                                                 any_order=True)
        api.neutron.subnet_get.assert_has_calls(subnet_get_calls,
                                                any_order=True)
        self.assertContains(res, "<a href=\"/project/security_services"
                                 "/%s\">%s</a>" % (sec_service.id,
                                                   sec_service.name), 1, 200)
        self.assertNoMessages()
        api_manila.share_network_security_service_list.assert_called_once_with(
            mock.ANY, share_net.id)
        api_manila.share_server_list.assert_called_once_with(
            mock.ANY, search_opts={'share_network_id': share_net.id})
        api_manila.share_network_get.assert_called_once_with(
            mock.ANY, share_net.id)

    def test_update_share_network(self):
        share_net = test_data.inactive_share_network
        formData = {
            'method': 'UpdateForm',
            'name': share_net.name,
            'description': share_net.description,
        }
        url = reverse('horizon:project:share_networks:share_network_update',
                      args=[share_net.id])
        self.mock_object(api_manila, "share_network_update")
        self.mock_object(
            api_manila, "share_network_get", mock.Mock(return_value=share_net))
        self.mock_object(
            api_manila, "share_network_security_service_list",
            mock.Mock(return_value=[test_data.sec_service]))
        self.mock_object(
            api_manila, "security_service_list",
            mock.Mock(return_value=[test_data.sec_service]))
        self.mock_object(
            policy, 'check',
            mock.Mock(side_effect=(lambda *args, **kwargs: True)))

        res = self.client.post(url, formData)

        self.assertRedirectsNoFollow(res, INDEX_URL)
        api_manila.share_network_security_service_list.assert_called_once_with(
            mock.ANY, share_net.id)
        api_manila.security_service_list.assert_has_calls([
            mock.call(mock.ANY),
            mock.call(mock.ANY, search_opts={'share_network_id': share_net.id})
        ])
        api_manila.share_network_get.assert_called_once_with(
            mock.ANY, share_net.id)
        api_manila.share_network_update.assert_called_once_with(
            mock.ANY,
            share_net.id,
            name=formData['name'],
            description=formData['description'])
        self.assertTrue(policy.check.called)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


from django.utils.translation import gettext_lazy as _
from horizon import exceptions
from horizon import forms
from horizon import messages
from horizon import workflows
from openstack_dashboard import api
from openstack_dashboard.api import base

from manila_ui.api import manila
from manila_ui.dashboards import utils


class CreateShareNetworkInfoAction(workflows.Action):
    share_network_name = forms.CharField(
        max_length=255, label=_("Name"), required=True)
    share_network_description = forms.CharField(
        widget=forms.Textarea, label=_("Description"), required=False)

    class Meta(object):
        name = ("Share Network")


class CreateShareNetworkInfoStep(workflows.Step):
    action_class = CreateShareNetworkInfoAction
    contributes = ("share_network_description",
                   "share_network_name")


class AddShareNetworkSubnetAction(workflows.MembershipAction):

    availability_zone = forms.ChoiceField(
        required=False,
        label=_('Availability Zone'),
        widget=forms.ThemableSelectWidget(attrs={
            'data-availability_zone': _('Availability Zone')}))

    neutron_net_id = forms.ChoiceField(
        required=False,
        label=_('Neutron Net'),
        widget=forms.ThemableSelectWidget(attrs={
            'class': 'switchable',
            'data-slug': 'neutron_net_id',
            'data-neutron_net_id': _('Neutron Net')}))

    class Meta(object):
        name = _("Subnet")
        help_text = _("Specify an Availability Zone or an existing subnet. "
                      "If no details are specified, "
                      "then a default subnet with a null Availability "
                      "Zone will be created automatically.")

    def __init__(self, request, context, *args, **kwargs):
        super().__init__(request, context, *args, **kwargs)

        self.fields['availability_zone'].choices = (
            self.get_availability_zone_choices(request)
        )
        self.neutron_enabled = base.is_service_enabled(request, 'network')
        if self.neutron_enabled:
            try:
                self.fields['neutron_net_id'].choices, networks = (
                    self.get_neutron_net_id_choices(request)
                )
            except Exception:
                msg = _('Unable to initialize neutron networks.')
                exceptions.handle(request, msg)
            try:
                self.get_neutron_subnet_id_choices(request, networks)
            except Exception:
                msg = _('Unable to initialize neutron subnets.')
                exceptions.handle(request, msg)

    def get_availability_zone_choices(self, request):
        availability_zone_choices = [('', _('None'))]

        for availability_zone in manila.availability_zone_list(request):
            availability_zone_choices.append(
                (availability_zone.id, availability_zone.name)
            )
        return availability_zone_choices

    def get_neutron_net_id_choices(self, request):
        net_choices = [('', _('None'))]

        networks = api.neutron.network_list(request)
        for network in networks:
            net_choices.append((utils.transform_dashed_name(network.id),
                                network.name_or_id))
        return net_choices, networks

    def get_neutron_subnet_id_choices(self, request, networks):
        for net in networks:
            subnet_field_name = (
                'subnet-choices-%s' % utils.transform_dashed_name(net.id))
            data_net_id = (
                'data-neutron_net_id-%s' % utils.transform_dashed_name(net.id))
            subnet_field = forms.ChoiceField(
                required=False,
                choices=(),
                label=_('Neutron Subnet'),
                widget=forms.ThemableSelectWidget(attrs={
                    'class': 'switched',
                    'data-switch-on': 'neutron_net_id',
                    data_net_id: _('Neutron Subnet')}))
            self.fields[subnet_field_name] = subnet_field
            subnet_choices = api.neutron.subnet_list(request,
                                                     network_id=net.id)
            self.fields[subnet_field_name].choices = [
                (choice.id, choice.name_or_id)
                for choice in subnet_choices]

    def hide_neutron_subnet_id_choices(self):
        self.fields['neutron_subnet_id'].choices = []
        self.fields['neutron_subnet_id'].widget = forms.HiddenInput()


class AddShareNetworkSubnetStep(workflows.Step):
    action_class = AddShareNetworkSubnetAction
    contributes = ("neutron_net_id", "neutron_subnet_id", "availability_zone")


class CreateShareNetworkWorkflow(workflows.Workflow):
    slug = "create_share_network"
    name = _("Create Share Network")
    finalize_button_name = _("Create Share Network")
    success_message = _('Created share network "%s".')
    failure_message = _('Unable to create share network "%s".')
    success_url = 'horizon:project:share_networks:index'
    default_steps = (CreateShareNetworkInfoStep, AddShareNetworkSubnetStep)
    wizard = True

    def handle(self, request, context):
        try:
            data = request.POST
            send_data = {'name': context['share_network_name']}
            if context['share_network_description']:
                send_data['description'] = context['share_network_description']
            neutron_net_id = context.get('neutron_net_id')
            if neutron_net_id:
                send_data['neutron_net_id'] = utils.transform_dashed_name(
                    neutron_net_id.strip())
                subnet_key = (
                    'subnet-choices-%s' % neutron_net_id.strip()
                )
                if data.get(subnet_key) is not None:
                    send_data['neutron_subnet_id'] = data.get(subnet_key)
            if context['availability_zone']:
                send_data['availability_zone'] = context['availability_zone']
            share_network = manila.share_network_create(request, **send_data)
            messages.success(request, _('Successfully created share'
                                        ' network: %s') % send_data['name'])
            return share_network
        except Exception:
            exceptions.handle(request, _('Unable to create share network.'))
            return False


class UpdateShareNetworkInfoAction(workflows.Action):
    name = forms.CharField(label=_("Name"),
                           max_length=255)

    description = forms.CharField(label=_("Description"),
                                  max_length=255,
                                  required=False)

    def __init__(self, request, *args, **kwargs):
        super(UpdateShareNetworkInfoAction, self).__init__(
            request, *args, **kwargs)
        share_network = manila.share_network_get(request,
                                                 self.initial['id'])
        self.fields['name'].initial = share_network.name
        self.fields['description'].initial = share_network.description

    class Meta(object):
        name = _("Share Network Info")
        help_text = _("From here you can update share network info. ")
        slug = "update-share_network_info"

    def clean(self):
        cleaned_data = super(UpdateShareNetworkInfoAction, self).clean()
        return cleaned_data


class UpdateShareNetworkInfoStep(workflows.Step):
    action_class = UpdateShareNetworkInfoAction
    contributes = ("description",
                   "name")


class AddSecurityServiceAction(workflows.MembershipAction):
    def __init__(self, request, *args, **kwargs):
        super(AddSecurityServiceAction, self).__init__(request,
                                                       *args,
                                                       **kwargs)
        err_msg = _('Unable to get the security services hosts')

        default_role_field_name = self.get_default_role_field_name()
        self.fields[default_role_field_name] = forms.CharField(required=False)
        self.fields[default_role_field_name].initial = 'member'

        field_name = self.get_member_field_name('member')
        self.fields[field_name] = forms.MultipleChoiceField(required=False)

        share_network_id = self.initial['id']
        security_services = manila.share_network_security_service_list(
            request, share_network_id)
        sec_services_initial = [sec_service.id for sec_service
                                in security_services]
        sec_services = []
        try:
            sec_services = manila.security_service_list(request)
        except Exception:
            exceptions.handle(request, err_msg)

        sec_services_choices = [(sec_service.id,
                                 sec_service.name or sec_service.id)
                                for sec_service in sec_services]
        self.fields[field_name].choices = sec_services_choices
        self.fields[field_name].initial = sec_services_initial

    class Meta(object):
        name = _("Security services within share network")
        slug = "add_security_service"


class AddSecurityServiceStep(workflows.UpdateMembersStep):
    action_class = AddSecurityServiceAction
    help_text = _("Add security services to share network.")
    available_list_title = _("Available security services")
    members_list_title = _("Selected security services")
    no_available_text = _("No security services found.")
    no_members_text = _("No security services selected.")
    show_roles = False
    depends_on = ("id",)
    contributes = ("security_services",)

    def contribute(self, data, context):
        if data:
            member_field_name = self.get_member_field_name('member')
            context['security_service'] = data.get(member_field_name, [])
        return context


class UpdateShareNetworkWorkflow(workflows.Workflow):
    slug = "update_share_network"
    name = _("Update Share Network")
    finalize_button_name = _("Update Share Network")
    success_message = _('Updated share network "%s".')
    failure_message = _('Unable to update share network "%s".')
    success_url = 'horizon:project:share_networks:index'
    default_steps = (UpdateShareNetworkInfoStep, AddSecurityServiceStep)

    def format_status_message(self, message):
        return message % self.context['name']

    def handle(self, request, context):
        try:
            manila.share_network_update(request, context['id'],
                                        description=context['description'],
                                        name=context['name'])
            sec_services = manila.security_service_list(request, search_opts={
                'share_network_id': context['id']})
            sec_services_old = set([sec_service.id
                                   for sec_service in sec_services])
            sec_services_new = set(context['security_service'])
            for sec_service in sec_services_new - sec_services_old:
                manila.share_network_security_service_add(request,
                                                          context['id'],
                                                          sec_service)
            for sec_service in sec_services_old - sec_services_new:
                manila.share_network_security_service_remove(request,
                                                             context['id'],
                                                             sec_service)
            return True
        except Exception:
            exceptions.handle(request, _('Unable to update share network.'))
            return False



# Copyright (c) 2015 Mirantis, Inc.
# All rights reserved.

# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from io import BytesIO

import ddt
from django.core.handlers import wsgi
from django.core.handlers.wsgi import LimitedStream
from django import forms as django_forms
from horizon import forms as horizon_forms
from unittest import mock

from manila_ui.dashboards.admin.share_types import forms
from manila_ui.tests import helpers as base


@ddt.ddt
class ManilaDashboardsAdminSharesUpdateShareTypeFormTests(base.APITestCase):

    def setUp(self):
        super(self.__class__, self).setUp()
        stream = LimitedStream(BytesIO(b"test"), 2)
        FAKE_ENVIRON = {'REQUEST_METHOD': 'GET', 'wsgi.input': stream}
        self.request = wsgi.WSGIRequest(FAKE_ENVIRON)

    def _get_form(self, initial):
        kwargs = {
            'prefix': None,
            'initial': initial,
        }
        return forms.UpdateShareType(self.request, **kwargs)

    @ddt.data(
        ({}, []),
        ({'foo': 'bar', 'quuz': 'zaab'}, ["foo=bar\r\n", "quuz=zaab\r\n"]),
    )
    @ddt.unpack
    def test___init__(self, extra_specs_dict_input, extra_specs_str_output):
        form = self._get_form({'extra_specs': extra_specs_dict_input})

        for expected_extra_spec in extra_specs_str_output:
            self.assertIn(expected_extra_spec, form.initial['extra_specs'])
        self.assertIn('extra_specs', list(form.fields.keys()))
        self.assertIsInstance(form.fields['extra_specs'],
                              horizon_forms.CharField)

    @mock.patch('horizon.messages.success')
    def test_handle_success_no_changes(self, mock_horizon_messages_success):
        initial = {'id': 'fake_id', 'name': 'fake_name', 'extra_specs': {}}
        form = self._get_form(initial)
        data = {'extra_specs': ''}

        result = form.handle(self.request, data)

        self.assertTrue(result)
        mock_horizon_messages_success.assert_called_once_with(
            mock.ANY, mock.ANY)

    @mock.patch('horizon.messages.success')
    def test_handle_success_only_set(self, mock_horizon_messages_success):
        initial = {
            'id': 'fake_id',
            'name': 'fake_name',
            'extra_specs': {'foo': 'bar'}
        }
        form = self._get_form(initial)
        data = {'extra_specs': 'foo=bar\r\n'}

        result = form.handle(self.request, data)

        self.assertTrue(result)
        mock_horizon_messages_success.assert_called_once_with(
            mock.ANY, mock.ANY)
        self.manilaclient.share_types.get.assert_called_with(
            initial['id'])
        self.manilaclient.share_types.get.return_value.set_keys.\
            assert_called_once_with({'foo': 'bar'})
        self.assertFalse(
            self.manilaclient.share_types.get.return_value.unset_keys.called)

    @mock.patch('horizon.messages.success')
    def test_handle_success_only_unset(self, mock_horizon_messages_success):
        initial = {
            'id': 'fake_id',
            'name': 'fake_name',
            'extra_specs': {'foo': 'bar'}
        }
        form = self._get_form(initial)
        data = {'extra_specs': 'foo\r\n'}
        share_types_get = self.manilaclient.share_types.get
        share_types_get.return_value.get_keys.return_value = {
            'foo': 'bar', 'quuz': 'zaab'}

        result = form.handle(self.request, data)

        self.assertTrue(result)
        mock_horizon_messages_success.assert_called_once_with(
            mock.ANY, mock.ANY)
        self.manilaclient.share_types.get.assert_has_calls([
            mock.call(initial['id'])])
        share_types_get.return_value.get_keys.assert_called_once_with()
        self.assertFalse(share_types_get.return_value.set_keys.called)
        share_types_get.return_value.unset_keys.assert_called_once_with(
            {'foo'})

    @mock.patch('horizon.messages.success')
    def test_handle_success_set_and_unset(self, mock_horizon_messages_success):
        initial = {
            'id': 'fake_id',
            'name': 'fake_name',
            'extra_specs': {'foo': 'bar'}
        }
        form = self._get_form(initial)
        data = {'extra_specs': 'foo\r\nquuz=zaab'}
        share_types_get = self.manilaclient.share_types.get
        share_types_get.return_value.get_keys.return_value = {'foo': 'bar'}

        result = form.handle(self.request, data)

        self.assertTrue(result)
        mock_horizon_messages_success.assert_called_once_with(
            mock.ANY, mock.ANY)
        self.manilaclient.share_types.get.assert_has_calls([
            mock.call(initial['id'])])
        share_types_get.return_value.get_keys.assert_called_once_with()
        share_types_get.return_value.set_keys.assert_called_once_with(
            {'quuz': 'zaab'})
        share_types_get.return_value.unset_keys.assert_called_once_with(
            {'foo'})

    def test_handle_validation_error(self):
        initial = {'id': 'fake_id', 'name': 'fake_name', 'extra_specs': {}}
        form = self._get_form(initial)
        form.api_error = mock.Mock()
        data = {'extra_specs': 'a b'}

        result = form.handle(self.request, data)

        self.assertFalse(result)
        form.api_error.assert_called_once_with(mock.ANY)

    @mock.patch('horizon.exceptions.handle')
    def test_handle_other_exception(self, mock_horizon_exceptions_handle):
        django_forms.ValidationError
        initial = {'id': 'fake_id', 'name': 'fake_name', 'extra_specs': {}}
        form = self._get_form(initial)
        data = {'extra_specs': None}

        result = form.handle(self.request, data)

        self.assertFalse(result)
        mock_horizon_exceptions_handle.assert_called_once_with(
            self.request, mock.ANY)

    @ddt.data(
        ('name_updated_1', 'description_updated', True),
        ('name_updated_2', 'description_updated', False),
        ('name_updated_3', None, True),
    )
    @ddt.unpack
    @mock.patch('horizon.messages.success')
    def test_update_share_type_name_description_public(
            self, name, description, is_public,
            mock_horizon_messages_success):
        initial = {
            'id': 'fake_id',
            'name': 'fake_name',
            'description': 'fake_description',
            'is_public': True,
            'extra_specs': {}
        }
        form = self._get_form(initial)
        data = {
            'name': name,
            'description': description,
            'is_public': is_public,
            'extra_specs': '',
        }
        result = form.handle(self.request, data)

        self.assertIs(True, result)
        mock_horizon_messages_success.assert_called_once_with(
            mock.ANY, mock.ANY)
        self.manilaclient.share_types.get.assert_called_once_with(
            initial['id'])
        self.manilaclient.share_types.get.return_value.update.\
            assert_called_once_with(name=data['name'],
                                    description=data['description'],
                                    is_public=data['is_public'])


@ddt.ddt
class ManilaDashboardsAdminSharesCreateShareTypeFormTests(base.APITestCase):

    def setUp(self):
        super(self.__class__, self).setUp()
        stream = LimitedStream(BytesIO(b"test"), 2)
        FAKE_ENVIRON = {'REQUEST_METHOD': 'GET', 'wsgi.input': stream}
        self.request = wsgi.WSGIRequest(FAKE_ENVIRON)

    def _get_form(self, **kwargs):
        return forms.CreateShareType(self.request, **kwargs)

    @mock.patch('horizon.messages.success')
    def test_create_share_type(self, mock_horizon_messages_success):
        form = self._get_form()
        data = {
            'extra_specs': '',
            'is_public': False,
            'spec_driver_handles_share_servers': 'True',
            'name': 'share',
            'description': ''
        }

        result = form.handle(self.request, data)

        self.assertTrue(result)
        self.manilaclient.share_types.create.assert_called_once_with(
            name=data['name'],
            spec_driver_handles_share_servers='true',
            description=data['description'],
            is_public=data["is_public"])
        mock_horizon_messages_success.assert_called_once_with(
            self.request, mock.ANY)

    @mock.patch('horizon.messages.success')
    def test_create_share_type_with_extra_specs_and_description(
            self, mock_horizon_messages_success):
        form = self._get_form()
        data = {'extra_specs': 'a=b \n c=d',
                'is_public': False,
                'spec_driver_handles_share_servers': 'True',
                'description': 'st_description',
                'name': 'share'}

        result = form.handle(self.request, data)

        self.assertTrue(result)

        set_keys = self.manilaclient.share_types.get.return_value.set_keys
        set_keys.assert_called_once_with(
            {'a': 'b', 'c': 'd'})
        self.manilaclient.share_types.create.assert_called_once_with(
            name=data['name'],
            spec_driver_handles_share_servers='true',
            description=data['description'],
            is_public=data["is_public"])
        mock_horizon_messages_success.assert_called_once_with(
            self.request, mock.ANY)

    @ddt.data(True, False)
    @mock.patch('horizon.messages.success')
    def test_public_share_type_creation(self,
                                        enable_public_share_type_creation,
                                        mock_horizon_messages_success):
        with self.settings(OPENSTACK_MANILA_FEATURES={
                'enable_public_share_type_creation':
                enable_public_share_type_creation}):
            form = self._get_form()

            data = {
                'extra_specs': '',
                'is_public': enable_public_share_type_creation,
                'spec_driver_handles_share_servers': 'True',
                'name': 'share',
                'description': '',
            }

            result = form.handle(self.request, data)

            self.assertTrue(result)
            self.assertEqual(
                enable_public_share_type_creation,
                form.enable_public_share_type_creation)
            if enable_public_share_type_creation:
                self.assertIn("is_public", form.fields)
                self.assertTrue(form.fields["is_public"])
            else:
                self.assertNotIn("is_public", form.fields)
            self.manilaclient.share_types.create.assert_called_once_with(
                name=data['name'],
                spec_driver_handles_share_servers='true',
                description=data['description'],
                is_public=enable_public_share_type_creation)
            mock_horizon_messages_success.assert_called_once_with(
                self.request, mock.ANY)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from django.utils.translation import gettext_lazy as _
from horizon import exceptions
from horizon import messages
from horizon import workflows

from manila_ui.api import manila
from manila_ui.dashboards.project.share_networks \
    import workflows as sn_workflows
from manila_ui.dashboards import utils


class AddShareNetworkSubnetStep(workflows.Step):
    action_class = sn_workflows.AddShareNetworkSubnetAction
    depends_on = ("share_network_id",)
    contributes = ("neutron_net_id", "neutron_subnet_id", "availability_zone")


class CreateShareNetworkSubnetsWorkflow(workflows.Workflow):
    slug = "create_share_network_subnet"
    name = _("Create Share Network Subnet")
    finalize_button_name = _("Create Share Network Subnet")
    success_message = _('')
    failure_message = _('')
    success_url = 'horizon:project:share_networks:index'
    default_steps = (AddShareNetworkSubnetStep,)

    def handle(self, request, context):
        try:
            data = request.POST
            share_network_id = self.context['share_network_id']
            share_network_name = manila.share_network_get(
                request, share_network_id).name
            send_data = {'share_network_id': share_network_id}

            neutron_net_id = context.get('neutron_net_id')
            if neutron_net_id:
                send_data['neutron_net_id'] = utils.transform_dashed_name(
                    neutron_net_id.strip())
                subnet_key = (
                    'subnet-choices-%s' % neutron_net_id.strip()
                )
                if data.get(subnet_key) is not None:
                    send_data['neutron_subnet_id'] = data.get(subnet_key)
            if context['availability_zone']:
                send_data['availability_zone'] = context['availability_zone']
            share_network = manila.share_network_subnet_create(request,
                                                               **send_data)

            messages.success(
                request,
                _('Successfully created share network subnet for: %s') %
                share_network_name)
            return share_network
        except Exception:
            exceptions.handle(request,
                              _('Unable to create share network subnet.'))
            return False
